/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  DeadFTP
 *  Copyright (C) 2000-2001 Brandon Lees
 */

/*
 * Initial main.c file generated by Glade. Edit as required.
 * Glade will not overwrite this file.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <libgnomeui/gnome-window-icon.h>
#include <glade/glade.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>

#include "globals.h"
#include "main.h"
#include "ftp.h"
#include "local.h"
#include "hostmanager.h"
#include "popups.h"
#include "layout.h"
#include "clipboard.h"
#include "fxp.h"
#include "remote.h"
#include "preferences.h"

/* Callbacks in this file */
void on_exit1_activate(GtkMenuItem * menuitem, gpointer user_data);
void on_mainwindow_destroy(GtkObject * object, gpointer user_data);
void on_about1_activate(GtkMenuItem * menuitem, gpointer user_data);
void on_cut1_activate(GtkMenuItem * menuitem, gpointer user_data);
void on_copy1_activate(GtkMenuItem * menuitem, gpointer user_data);
void on_paste1_activate(GtkMenuItem * menuitem, gpointer user_data);
void on_clear1_activate(GtkMenuItem * menuitem, gpointer user_data);
void on_deadftp_manual1_activate(GtkMenuItem * menuitem, gpointer user_data);
void on_finish_exit_toggled(GtkCheckMenuItem * check_menu_item);
void on_finish_disconnect_toggled(GtkCheckMenuItem * check_menu_item);
void on_finish_idle_toggled(GtkCheckMenuItem * check_menu_item);

/* Local functions */
static void console_log(console_level_t, const char *);

/* Variables */
mainwindow_t deadftp;
static FILE *console_log_file = NULL;

int main(int argc, char *argv[])
{
  GladeXML *xml;

#ifdef ENABLE_NLS
  bindtextdomain(PACKAGE, PACKAGE_LOCALE_DIR);
  textdomain(PACKAGE);
#endif

  umask(022);

  g_thread_init(NULL);

  gnome_init("deadftp", VERSION, argc, argv);

  gnome_window_icon_set_default_from_file(DATADIR "/deadftp/skull-icon.png");
  gnome_window_icon_init();

  glade_gnome_init();

  gnome_config_push_prefix("/deadftp/");

  load_preferences();

#ifdef CALL_PTHREAD_SETCONCURRENCY
  pthread_setconcurrency(3);
#endif /* CALL_PTHREAD_SETCONCURRENCY */

  /* Try to load ../deadftp.glade for me when I'm in the src dir */
  if (g_file_exists("../pixmaps/deadftp.glade"))
    strncpy(deadftp.gladefile, "../pixmaps/deadftp.glade", REGULAR_BUFFER);
  else
    strncpy(deadftp.gladefile, DATADIR "/deadftp/deadftp.glade", REGULAR_BUFFER);

  xml = glade_xml_new(deadftp.gladefile, "mainwindow");

  if (!xml) {
    g_error(_("Couldn't load the interface!"));
    exit(1);
  }

  deadftp.window = glade_xml_get_widget(xml, "mainwindow");

  STORE_WIDGET(deadftp, quick_hostname);
  STORE_WIDGET(deadftp, quick_hostname_combo);
  STORE_WIDGET(deadftp, quick_port);
  STORE_WIDGET(deadftp, quick_port_combo);
  STORE_WIDGET(deadftp, quick_username);
  STORE_WIDGET(deadftp, quick_username_combo);
  STORE_WIDGET(deadftp, quick_password);

  STORE_WIDGET(deadftp, local_textcombo);
  STORE_WIDGET(deadftp, local_combo);
  STORE_WIDGET(deadftp, local_up);
  STORE_WIDGET(deadftp, local);
  gtk_clist_set_row_height(GTK_CLIST(deadftp.local), 20);

  STORE_WIDGET(deadftp, remote_scroll);
  STORE_WIDGET(deadftp, host_scroll);

  STORE_WIDGET(deadftp, remote_textcombo);
  STORE_WIDGET(deadftp, remote_combo);
  STORE_WIDGET(deadftp, remote_up);
  STORE_WIDGET(deadftp, remote);
  gtk_clist_set_row_height(GTK_CLIST(deadftp.remote), 20);

  STORE_WIDGET(deadftp, queue);
  gtk_clist_set_row_height(GTK_CLIST(deadftp.queue), 20);
  STORE_WIDGET(deadftp, console);

  STORE_WIDGET(deadftp, status_state);
  STORE_WIDGET(deadftp, status_progress);
  STORE_WIDGET(deadftp, status_speed);
  STORE_WIDGET(deadftp, status_elapsed);
  STORE_WIDGET(deadftp, status_remaining);
  STORE_WIDGET(deadftp, status_queue);

  STORE_WIDGET(deadftp, finish_idle);
  STORE_WIDGET(deadftp, finish_disconnect);
  STORE_WIDGET(deadftp, finish_exit);

  /* Although in the main window, host_ctree is used by hostmanager */
  STORE_WIDGET(hostmanager, host_ctree);

  glade_xml_signal_autoconnect(xml);

  gtk_object_unref(GTK_OBJECT(xml));

  init_hostmanager();

  hm_redisplay_ctree();

  gtk_combo_disable_activate(GTK_COMBO(deadftp.local_combo));
  gtk_combo_disable_activate(GTK_COMBO(deadftp.remote_combo));
  gtk_combo_disable_activate(GTK_COMBO(deadftp.quick_hostname_combo));
  gtk_combo_disable_activate(GTK_COMBO(deadftp.quick_port_combo));
  gtk_combo_disable_activate(GTK_COMBO(deadftp.quick_username_combo));
  gtk_clist_set_reorderable(GTK_CLIST(deadftp.queue), TRUE);

  switch (deadftp_prefs.finish_action) {
  case FINISH_IDLE:
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(deadftp.finish_idle), TRUE);
    break;
  case FINISH_DISCONNECT:
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(deadftp.finish_disconnect),
				   TRUE);
    break;
  case FINISH_EXIT:
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(deadftp.finish_exit), TRUE);
    break;
  }

  init_remote_dir_layout();
  init_local_dir_layout();
  init_queue_layout();

  ftp_init();
  remote_init();
  change_local();
  init_popups();
  init_clipboard_monitor();
  init_fxp();

  /* Change directory to the download_directory to start */
  chdir(deadftp_prefs.download_directory);

  gtk_main();

  gtk_widget_destroy(deadftp.window);
  save_preferences();

  return 0;
}

void on_exit1_activate(GtkMenuItem * menuitem, gpointer user_data)
{
  ftp_exit(0);
}

void on_mainwindow_destroy(GtkObject * object, gpointer user_data)
{
  ftp_exit(0);
}

void on_about1_activate(GtkMenuItem * menuitem, gpointer user_data)
{
  GladeXML *xml;

  xml = glade_xml_new(deadftp.gladefile, "aboutDialog");
  glade_xml_signal_autoconnect(xml);
}

void on_cut1_activate(GtkMenuItem * menuitem, gpointer user_data)
{

}
void on_copy1_activate(GtkMenuItem * menuitem, gpointer user_data)
{

}
void on_paste1_activate(GtkMenuItem * menuitem, gpointer user_data)
{

}
void on_clear1_activate(GtkMenuItem * menuitem, gpointer user_data)
{

}

void on_deadftp_manual1_activate(GtkMenuItem * menuitem, gpointer user_data)
{
  GnomeHelpMenuEntry help_entry = { "deadftp", "index.html" };
  gnome_help_display(NULL, &help_entry);
}

void print_console(console_level_t level, char *message, ...)
{
  va_list args;
  char buffer[REGULAR_BUFFER], *p;
  GdkColor *color;

  g_return_if_fail(message != NULL);

  va_start(args, message);
  g_vsnprintf(buffer, REGULAR_BUFFER, message, args);
  va_end(args);

  switch (level) {
  case CONSOLE_LOCAL:
    color = &deadftp_prefs.local;
    break;
  case CONSOLE_REMOTE:
    color = &deadftp_prefs.remote;
    break;
  case CONSOLE_INFO:
    color = &deadftp_prefs.info;
    break;
  case CONSOLE_ERROR:
    color = &deadftp_prefs.error;
    break;
  default:
    g_assert_not_reached();
    return;
  }

  /* change \r\n to just \n to stop weird boxes in console display */
  p = strchr(buffer, '\r');
  if (p) {
    *p++ = '\n';
    *p = '\0';
  }

  /* If logging is turned on, print this to the logfile */
  if (deadftp_prefs.logging)
    console_log(level, buffer);

  /* If print_console is called from a signal handler then the lock has already been acquired
   * otherwhise it hasn't been.  This poses a strange problem since if we try to lock it from
   * a signal handler we will go into a deadlock while if we try to get around this it will make
   * the lock useless in other places*/
/*  GDK_THREADS_ENTER(); */
  gtk_text_insert(GTK_TEXT(deadftp.console), NULL, color, NULL, buffer, -1);
/*  GDK_THREADS_LEAVE(); */
}

void console_log_init()
{
  /* If the log file is already open close it so we don't
   * leak file descriptors */
  if (console_log_file) {
    fclose(console_log_file);
    console_log_file = NULL;
  }

  if (!deadftp_prefs.logging)
    return;

  /* Open the logfile */
  console_log_file = fopen(deadftp_prefs.log_filename, "a");
  if (!console_log_file) {
    GtkWidget *dialog =
      gnome_warning_dialog_parented(_("Could not open logfile.  Session will not be logged"),
				    GTK_WINDOW(deadftp.window));
    gtk_widget_show(dialog);
  }
}

void console_log(console_level_t level, const char *message)
{
  if (!deadftp_prefs.logging || !console_log_file)
    return;

  switch (level) {
  case CONSOLE_LOCAL:
    fputs("--> ", console_log_file);
    break;
  case CONSOLE_REMOTE:
    fputs("<-- ", console_log_file);
    break;
  case CONSOLE_INFO:
    fputs("??? ", console_log_file);
    break;
  case CONSOLE_ERROR:
    fputs("!!! ", console_log_file);
    break;
  default:
    g_assert_not_reached();
    return;
  }

  fputs(message, console_log_file);
}

void on_finish_exit_toggled(GtkCheckMenuItem * check_menu_item)
{
  if (GTK_CHECK_MENU_ITEM(check_menu_item)->active)
    deadftp_prefs.finish_action = FINISH_EXIT;
}

void on_finish_disconnect_toggled(GtkCheckMenuItem * check_menu_item)
{
  if (GTK_CHECK_MENU_ITEM(check_menu_item)->active)
    deadftp_prefs.finish_action = FINISH_DISCONNECT;
}

void on_finish_idle_toggled(GtkCheckMenuItem * check_menu_item)
{
  if (GTK_CHECK_MENU_ITEM(check_menu_item)->active)
    deadftp_prefs.finish_action = FINISH_IDLE;
}
