/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  DeadFTP
 *  Copyright (C) 2000-2001 Brandon Lees
 */

#ifndef	_QUEUE_H_DEAD_
#define	_QUEUE_H_DEAD_

#include <pthread.h>

typedef unsigned long queueid_t;
struct fxp_desc;
struct ftp_server;

typedef struct queue_item {
  pthread_mutex_t lock;

  /* Host information */
  char hostname[SMALL_BUFFER];
  int port;
  char username[SMALL_BUFFER];
  char password[SMALL_BUFFER];

  /* file information */
  char remote_file[SMALL_BUFFER];
  char remote_dir[SMALL_BUFFER];
  char local_file[SMALL_BUFFER];
  char local_dir[SMALL_BUFFER];

  unsigned long mode;
  unsigned long size;
  unsigned long total_trans;
  time_t trans_start;
  int resume;

  /* last modified time for remote machine.  Used in
   * the new resume dialog */
  unsigned long remote_modified;

  unsigned int flags;

  struct fxp_desc *fxp;
  /* if this is a slave fxp then this is the master qid and the MASTER_ID bit
   * is set */
  queueid_t queueid;
} queue_item_t;

typedef struct queue_info {
  time_t transfer_time;

  unsigned long upsize;
  unsigned long downsize;
  double upspeed;
  double downspeed;

  resume_t resume_action;

  queue_item_t *current;
} queue_info_t;

#define QUEUE_SIZE(q)	      ((q).upsize + (q).downsize)
#define QUEUE_CURRENT(q)      ((q).current)

extern GList *ftp_queue, *ftp_queue_current;
extern queueid_t queueid;

queueid_t add_queue(const struct ftp_server *server, struct afile *file,
		    const char *local_dir, const char *local_file, gboolean send);
queueid_t add_queue_item(queue_item_t * q);
queueid_t insert_queue(const struct ftp_server *server, struct afile *file,
		       const char *local_dir, const char *local_file, gboolean send,
		       int position);
queueid_t insert_queue_item(queue_item_t * item, int position);
void queue_start_item(queue_item_t * q);
void queue_start(void);
void queue_next(gboolean);
void queue_go_id(gboolean slave, queueid_t qid);
void finish_queue(void);
void queue_error(const char *);
void queue_abort(void);
queueid_t queue_add_fxp(struct fxp_desc *fxp, const char *hostname, int port,
			const char *username, const char *password,
			const char *remote_dir, const char *remote_file,
			unsigned long size, queueid_t queueid);
gboolean queue_check_fxp_go(void);
void queue_item_delete_id(gboolean slave, queueid_t qid);
void queue_calculate_size(void);
void queue_add_url(const char *url);
void queue_item_delete(queue_item_t * q);

#define	QUEUE_STATUS_NORMAL	0
#define	QUEUE_STATUS_PAUSE	1
#define	QUEUE_STATUS_ACTIVE	2
#define	QUEUE_STATUS_ERROR	3

#define	NUM_QUEUE_STATUS	4

#define	QUEUE_STATUS_MASK	0x7
#define	QUEUE_SET_STATUS(item, status)		((item)->flags &= ~QUEUE_STATUS_MASK, \
                                                 (item)->flags |= ((status) & 0x7))
#define	QUEUE_GET_STATUS(item)			((item)->flags & QUEUE_STATUS_MASK)

#define	QUEUE_GET_FLAG(item, flag)		((item)->flags & (flag))
#define	QUEUE_SET_FLAG(item, flag)		((item)->flags |= (flag))
#define	QUEUE_CLEAR_FLAG(item, flag)		((item)->flags &= ~(flag))

/* first three bits are used by status so flags start with the fourth bit */
#define	QUEUE_FLAG_SEND		(1 << 3)
#define	QUEUE_FLAG_RESUME	(1 << 4)
/* START_TRANS is set when check_resume is called from queue_go so when the user has clicked either
 * resume or overwrite in the dialog it knows to start the transfer instead of adding it to the queue
 */
#define	QUEUE_FLAG_START_TRANS	(1 << 5)
/* Whether or not we have asked the user to resume yet */
#define	QUEUE_FLAG_RESUME_ASKED	(1 << 6)
#define QUEUE_FLAG_MASTER_ID    (1 << 7)
#define QUEUE_FLAG_DIRECTORY    (1 << 8)
#define QUEUE_FLAG_CREATE_DEST  (1 << 9)

#define QUEUE_ITEM_LOCK(item)			pthread_mutex_lock(&(item)->lock)
#define QUEUE_ITEM_UNLOCK(item)			pthread_mutex_unlock(&(item)->lock)

#define QUEUE_OK_TRANSFER(x)    ((QUEUE_GET_STATUS(x) != QUEUE_STATUS_PAUSE) && \
                                 (QUEUE_GET_STATUS(x) != QUEUE_STATUS_ERROR))
#endif /* _QUEUE_H_DEAD_ */
