/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  DeadFTP
 *  Copyright (C) 2000-2001 Brandon Lees
 */

#ifndef	_SOCKET_H_DEAD_
#define	_SOCKET_H_DEAD_

#include <netinet/in.h>

typedef struct socket_ctx_s socket_ctx_t;

typedef struct {
  enum {
    PROXY_NONE, PROXY_USER1, PROXY_USER2, PROXY_USER_AUTH1, PROXY_USER_AUTH2,
    PROXY_SOCKS4, PROXY_SOCKS5
  } type;

  enum {
    SOCK_CONNECT, SOCK_BIND
  } sock_type;

  unsigned long addr;
  unsigned short port;

  char buffer[SMALL_BUFFER];
  int bufferlen;
  gint watchid;
  void (*callback) (socket_ctx_t *, gboolean);
} proxy_info_t;

typedef void (*socket_callback_func) (socket_ctx_t *, gpointer);

struct socket_ctx_s {
  char *hostname;
  int port;
  struct sockaddr_in addr;

  int sockfd;
  gint watchid;
  pthread_t thread;

  socket_callback_func callback;
  gpointer data;

  proxy_info_t *proxy;
};

void socket_connect(socket_ctx_t * sock, const char *hostname, int port,
		    socket_callback_func callback, gpointer data);
void socket_addr_connect(socket_ctx_t * sock, unsigned long addr, int port,
			 socket_callback_func callback, gpointer data);
void socket_new_server(socket_ctx_t * sock, int port, socket_callback_func callback,
		       gpointer data);
char *socket_make_string(socket_ctx_t *);
void socket_accept_connection(socket_ctx_t *, socket_callback_func callback,
			      gpointer data);
int socket_set_nonblock(socket_ctx_t *, int nonblock);
void socket_set_data(socket_ctx_t *);
void socket_error(socket_ctx_t * sock);
void socket_finish(socket_ctx_t * sock);
void socket_free(socket_ctx_t * sock);
void socket_disconnect(socket_ctx_t * sock);
socket_ctx_t *socket_new(void);

#endif /* _SOCKET_H_DEAD_ */
